#!/usr/bin/perl


BEGIN {
    die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
    unshift (@INC, "$ENV{'GSDLHOME'}/perllib");
    unshift (@INC, "$ENV{'GSDLHOME'}/perllib/plugins");
}


use parsargv;
use printusage;
my $unicode_list =
    [ { 'name' => "auto",
	'desc' => "{BasPlug.input_encoding.auto}" },
      { 'name' => "ascii",
	'desc' => "{BasPlug.input_encoding.ascii}" },
      { 'name' => "utf8",
	'desc' => "{BasPlug.input_encoding.utf8}" },
      { 'name' => "unicode",
	'desc' => "{BasPlug.input_encoding.unicode}" } ];

my $arguments = 
    [ { 'name' => "input_encoding",
	'desc' => "{explode.encoding}",
	'type' => "enum",
	'deft' => "auto",
	'list' => $unicode_list,
	'reqd' => "no" },
      { 'name' => "metadata_set",
	'desc' => "{explode.metadata_set}",
	'type' => "string",
	'reqd' => "no",
	'hiddengli' => "yes"},
      { 'name' => "plugin",
	'desc' => "{explode.plugin}",
	'type' => "string",
	'reqd' => "yes",
	'hiddengli' => "yes"},
      { 'name' => "document_field",
	'desc' => "{explode.document_field}",
	'type' => "string",
	'reqd' => "no"},
       { 'name' => "document_prefix",
	'desc' => "{explode.document_prefix}",
	'type' => "string",
	'reqd' => "no"},
      { 'name' => "document_suffix",
	'desc' => "{explode.document_suffix}",
	'type' => "string",
	'reqd' => "no"},
      { 'name' => "filename_field",
	'desc' => "{explode.filename_field}",
	'type' => "string",
	'reqd' => "no"}     
      ];
	
my $options = { 'name' => "explode_metadata_database.pl",
		'desc' => "{explode.desc}",
		'args' => $arguments };

	    
sub main
{
    my ($encoding, $metadata_set, $plugin, $filename_field, 
	$document_field, $document_prefix, $document_suffix);

    my $xml = 0;
    # Parse command line arguments
    if (!parsargv::parse(\@ARGV, 
			 'language/.*/', \$language,
			 'input_encoding/.*/auto', \$encoding,
			 'metadata_set/.*/', \$metadata_set,
			 'plugin/.*/', \$plugin,
			 'filename_field/.*/', \$filename_field,
			 'document_field/.*/', \$document_field,
			 'document_prefix/.*/', \$document_prefix,
			 'document_suffix/.*/', \$document_suffix,
			 q^xml^, \$xml)) {
	&PrintUsage::print_txt_usage($options, "{explode.params}");
	die "\n";
    }
    
    # If $language has been specified, load the appropriate resource bundle
    # (Otherwise, the default resource bundle will be loaded automatically)
    if ($language) {
	&gsprintf::load_language_specific_resource_bundle($language);
    }

    if ($xml) {
        &PrintUsage::print_xml_usage($options);
	print "\n";
	return;
    }

    # The metadata database filename is the first value that remains after the options have been parsed out
    my $filename = $ARGV[0];
    if (!defined $filename || $filename !~ /\w/) { 
	&PrintUsage::print_txt_usage($options, "{explode.params}");
	print STDERR "You need to specify a filename";
	die "\n";
    }
    # check that file exists
    if (!-e $filename) {
	print STDERR "File $filename doesn't exist...\n";
	die "\n";
    }
    # check required options
    if (!defined $plugin || $plugin !~ /\w/) {
	&PrintUsage::print_txt_usage($options, "{explode.params}");
	print STDERR "You need to specify a plugin";
	die "\n";
    }
    
    # check metadata set
    if (defined $metadata_set && $metadata_set =~ /\w/) {
	$metadata_set .= ".";
    } else {
	$metadata_set = "";
    }

    #check filename field
    if (defined $fileanme_field && $filenmae_field eq "") {
	undef $filename_field;
    }
    my $plugobj;
    require "$plugin.pm";
    eval ("\$plugobj = new $plugin()");
    die "$@" if $@;

    # ...and initialize it
    $plugobj->init(1, "STDERR", "STDERR");
    
    if ($encoding eq "auto") {
	$plugobj->{'input_encoding'} = $encoding;   
	(my $language, $encoding) = $plugobj->textcat_get_language_encoding ($filename);
    }
    # Use the plugin's read_file function to avoid duplicating code
    $plugobj->read_file($filename, $encoding, undef, \$text);

    # Create a directory to store the document files...
    my ($documents_directory) = ($filename =~ /(.*)\.[^\.]+$/);
    if (-d $documents_directory) {
	die "Error: document directory $documents_directory already exists (bailing).\n";
    }
    &util::mk_dir($documents_directory);

    # ...and a metadata.xml file for the document metadata (extracted from the database)
    $documents_metadata_xml_file = &util::filename_cat($documents_directory, "metadata.xml");
    if (-e $documents_metadata_xml_file) {
	die "Error: document metadata.xml file $documents_metadata_xml_file already exists (bailing).\n";
    }

    # Start the metadata.xml file
    open(METADATA_XML_FILE, ">$documents_metadata_xml_file");
    print METADATA_XML_FILE
	"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n" .
	"<!DOCTYPE DirectoryMetadata SYSTEM \"http://greenstone.org/dtd/DirectoryMetadata/1.0/DirectoryMetadata.dtd\">\n" .
	"<DirectoryMetadata>\n";

    # Split the text into records, using the plugin's split_exp
    my $split_exp = $plugobj->{'split_exp'};
    my @metadata_records = split(/$split_exp/, $text);
    print STDERR "Number of records: " . @metadata_records . "\n";

    # Write the metadata from each record to the metadata.xml file
    my $record_number = 0;
    foreach $record_text (@metadata_records) {
	
	# Use the plugin's process function to avoid duplicating code
	my $doc_obj = new doc($filename, "nonindexed_doc");
	$plugobj->process(\$record_text, undef, undef, $filename, undef, $doc_obj, 0);
	# Get all the metadata assigned to this record
	my $record_metadata = $doc_obj->get_all_metadata($doc_obj->get_top_section());
	my $document_file;
	
	# try to get a doc to attach the metadata to
	if (defined $document_field) {
	    foreach my $pair (@$record_metadata) {
		my ($field, $value) = (@$pair);
		
		# Does this metadata element specify a document to obtain?
		if ($field eq $document_field) {
		    my $document_file_full = $document_prefix . $value . $document_suffix;
		    $document_file = &obtain_document($self, $document_file_full, $documents_directory);
		}
	    }
	}
	# do we need to create a dummy doc??
	if (not defined $document_file) {
	    # try to get a file name
	    if (defined $filename_field) {
		my $meta = $doc_obj->get_metadata_element($doc_obj->get_top_section(), $filename_field);
		if (defined $meta) {
		    $meta =~ s/&\w{1,10};//g; # remove entities
		    $document_file = "$meta.nul";
		    my $num = 0;
		    while (-e "$documents_directory/$document_file") {
			$num++;
			$document_file = "$meta$num.nul";
		    }
		} else {
		    $record_number = $record_number + 1;
		    $document_file = sprintf("%4.4d", $record_number) . ".nul";
		}
	    } else {
		$record_number = $record_number + 1;
		$document_file = sprintf("%4.4d", $record_number) . ".nul";
	    }
	    open(DUMMY_FILE, ">$documents_directory/$document_file");
	    close(DUMMY_FILE);

	}
	

	&write_metadata_xml_file_entry(METADATA_XML_FILE, $document_file, $record_metadata, $metadata_set);
    }

    # Finish and close the metadata.xml file
    print METADATA_XML_FILE "\n</DirectoryMetadata>\n";
    close(METADATA_XML_FILE);

    # Explode means just that: the original file is deleted
    &util::rm($filename);
}


sub write_metadata_xml_file_entry
{
    my $metadata_xml_file = shift(@_);
    my $file_name = shift(@_);
    my $record_metadata = shift(@_);
    my $meta_prefix = shift(@_);
    
    # Make $file_name XML-safe
    $file_name =~ s/</&lt;/g;
    $file_name =~ s/>/&gt;/g;

    # Convert $file_name into a regular expression that matches it
    $file_name =~ s/\./\\\./g;
    $file_name =~ s/\(/\\\(/g;
    $file_name =~ s/\)/\\\)/g;
    $file_name =~ s/\{/\\\{/g;
    $file_name =~ s/\}/\\\}/g;
    $file_name =~ s/\[/\\\[/g;
    $file_name =~ s/\]/\\\]/g;
    
    print $metadata_xml_file
	"\n" .
        "  <FileSet>\n" .
	"    <FileName>$file_name</FileName>\n" .
	"    <Description>\n";

    foreach my $pair (@$record_metadata) {
	my ($field, $value) = (@$pair);

	# We're only interested in metadata from the database
	next if ($field eq "lastmodified");
	next if ($field eq "gsdlsourcefilename");
	next if ($field eq "gsdldoctype");
	next if ($field eq "FileFormat");

	# Ignore the ^all metadata, since it will be invalid if the source metadata is changed
	next if ($field =~ /\^all$/);  # ISISPlug specific!

	# Make $value XML-safe
	$value =~ s/</&lt;/g;
	$value =~ s/>/&gt;/g;

	# we are not allowed & in xml except in entities. 
	# if there are undefined entities then parsing will also crap out.
	# should we be checking for them too?
	# this may not get all possibilities
	$value =~ s/&([^;\s]*(\s|$))/&amp;$1/g;

	print $metadata_xml_file "      <Metadata mode=\"accumulate\" name=\"$meta_prefix$field\">$value</Metadata>\n";
    }

    print $metadata_xml_file
	"    </Description>\n" .
	    "  </FileSet>\n";
}

sub obtain_document
{
    my $self = shift(@_);
    my $document_file_full = shift(@_);
    my $documents_directory = shift(@_);

    my $outhandle = $self->{'outhandle'};
    print $outhandle "Obtaining document file $document_file_full...\n"
	if ($self->{'verbosity'} > 1);

    my $document_file_name;
    my $local_document_file;

    # Document specified is on the web
    if ($document_file_full =~ /^http:/ || $document_file_full =~ /^ftp:/) {
	$document_file_full =~ /([^\/]+)$/;
	$document_file_name = $1;
	$local_document_file = &util::filename_cat($documents_directory, $document_file_name);

	my $wget_options = "--quiet";
	$wget_options = "--verbose" if ($self->{'verbosity'} > 2);
	$wget_options .= " --timestamping";  # Only re-download files if they're newer
	`wget $wget_options $document_file_full --output-document $local_document_file`;
    }
    # Document specified is on the disk
    else {
	my $dir_sep = &util::get_os_dirsep();
	$document_file_full =~ /(.+$dir_sep)?(.*)$/;
	$document_file_name = $2;
	$local_document_file = &util::filename_cat($documents_directory, $document_file_name);

	&util::cp($document_file_full, $documents_directory);
    }

    # Check the document was obtained successfully
    if (!-e $local_document_file) {
	print STDERR "WARNING: Could not obtain document file $document_file_full\n";
	return undef;
    }

    return $document_file_name;
}

&main(@ARGV);
